import choco.kernel.solver.search.integer.AbstractIntVarSelector;
import choco.kernel.solver.variables.integer.IntDomainVar;
import choco.cp.solver.CPSolver;
import choco.kernel.solver.Solver;

public class MinSlackHeuristic extends AbstractIntVarSelector {
    
    int n;                   // number of decision variables
    IntDomainVar[] decision; // 0/1 decision variables
    IntDomainVar[] op1;      // start time of op1
    IntDomainVar[] op2;      // start time of op2
    int[] duration1;         // duration of op1
    int[] duration2;         // duration of op2

    public MinSlackHeuristic(Solver solver,Decision[] decisions){
	n         = decisions.length;
	decision  = new IntDomainVar[n];
	op1       = new IntDomainVar[n];
	op2       = new IntDomainVar[n];
	duration1 = new int[n];
	duration2 = new int[n];
	for (int i=0;i<n;i++){
	    decision[i]  = solver.getVar(decisions[i].d);
	    op1[i]       = solver.getVar(decisions[i].op_i.start);
	    op2[i]       = solver.getVar(decisions[i].op_j.start);
	    duration1[i] = decisions[i].op_i.duration;
	    duration2[i] = decisions[i].op_j.duration;
	}
    }

    public IntDomainVar selectIntVar(){
	int minSlack = Integer.MAX_VALUE;
	int slack = 0;
	IntDomainVar minSlackVar = null;
	for (int i=0;i<n;i++)
	    if (!decision[i].isInstantiated()){
		slack = slack(i);
		if (slack < minSlack){minSlackVar = decision[i];minSlack = slack;}
	    }
	return minSlackVar;
    }
    //
    // select the uninstantiated variable with minimum slack
    //

    private int slack(IntDomainVar op_i,int d_i,IntDomainVar op_j,int d_j){
	return op_j.getSup() - Math.max(op_i.getInf() + d_i,op_j.getInf());
    }
    //
    // slack if op_i before op_j
    //

    private int slack(int i){
	return Math.max(slack(op1[i],duration1[i],op2[i],duration2[i]),
			slack(op2[i],duration2[i],op1[i],duration1[i]));
    }
    //
    // get slack of ith decision variable
    //

}
